//******************************************************************************
// TotalToolbar functions.
//******************************************************************************

var TT = {
  get windowType() {
    return document.documentElement.getAttribute('windowtype');
  },

  get TabsToolbar() {
    return document.getElementById("TabsToolbar");
  },

  get isTreeStyleTabsEnabled() {
    return "TreeStyleTabBrowser" in window;
  },

  get isTreeStyleTabsModeVertical() {
    return this.TabsToolbar.getAttribute("treestyletab-mode") == "vertical";
  },

  /* Initialization function */
  init: function init() {
    if (this.windowType != "navigator:browser")
      // Only initialize if in main window.
      return;

    BrowserCustomizeToolbar = function BrowserCustomizeToolbar() {
      // Disable the toolbar context menu items
      var menubar = document.getElementById("main-menubar");
      for (var i = 0; i < menubar.childNodes.length; ++i)
        menubar.childNodes[i].setAttribute("disabled", true);

      var cmd = document.getElementById("cmd_CustomizeToolbars");
      cmd.setAttribute("disabled", "true");

      var splitter = document.getElementById("urlbar-search-splitter");
      if (splitter)
        splitter.parentNode.removeChild(splitter);

      // Disable tabs.
      for (var i = 0; i < gBrowser.tabContainer.childNodes.length; ++i)
        gBrowser.tabContainer.childNodes[i].setAttribute("disabled", true);

      var menu = document.getElementById("tt-toolbar-properties");
      if (menu)
        menu.setAttribute("disabled", true);

      setTimeout(function() { TT.setToolbarsForTST(true) }, 100);

      CombinedStopReload.uninit();

      PlacesToolbarHelper.customizeStart();
      BookmarksMenuButton.customizeStart();

      var customizeURL = "chrome://global/content/customizeToolbar.xul";
      var customizeTab = gBrowser.addTab(customizeURL,
                                         {ownerTab: gBrowser.selectedTab});
      gBrowser.moveTabTo(customizeTab, gBrowser.selectedTab._tPos + 1);
      gBrowser.selectedTab = customizeTab;
      gBrowser.selectedTab.setAttribute("customizetoolbartab", true);
    }

    var fx_BrowserFullScreen = BrowserFullScreen;
    BrowserFullScreen = function BrowserFullScreen() {
      fx_BrowserFullScreen();
      TT.BrowserFullScreen();
    }

    var fx_BrowserToolboxCustomizeDone = BrowserToolboxCustomizeDone;
    gNavToolbox.customizeDone = function BrowserToolboxCustomizeDone() {
      fx_BrowserToolboxCustomizeDone();

      // Enable tabs.
      for (var i = 0; i < gBrowser.tabContainer.childNodes.length; ++i)
        gBrowser.tabContainer.childNodes[i].removeAttribute("disabled");

      document.getElementById("tt-toolbar-properties").removeAttribute("disabled");
      TT.adjustToolboxWidth(true);
      TT.setFixedToolbarEmptyState();

      setTimeout(function() { TT.setToolbarsForTST(false); }, 500);
    }

    gNavToolbox._newElementCount = 0;

    // Label der Toolbareinstellungen tauschen, wenn der Browser im RTL-Modus
    // ist (Sidebar rechts).
    let browser = document.getElementById("browser");
    let wStyle = window.getComputedStyle(browser, null);
    if (wStyle.direction == "rtl") {
      let str_left        = document.getElementById("tt-left");
      let str_sidebarleft = document.getElementById("tt-sidebarleft");
      let str_right       = document.getElementById("tt-right");
      str_left.setAttribute("label", str_right.label);
      str_sidebarleft.setAttribute("label", str_sidebarleft.label);
      str_right.setAttribute("label", str_left.label);
    }

    TT.setTabsToolbar();

    TT.adjustToolboxWidth(true);

    TT.setFixedToolbarEmptyState();

    // Support TreeStyleTabs (because it's great).  The above and below browser
    // toolbars need to be moved into our binding toolboxes, if there are
    // toolbars there and TST vertical mode is on.
    TT.setToolbarsForTST(false);
  },

  setTabsToolbar: function setTabsToolbar(aToolboxId) {
    if (this.isTreeStyleTabsEnabled)
      // Don't move the TabsToolbar if TreeStyleTabs is installed.
      return;

    let toolbar = this.TabsToolbar;
    let ownertoolboxId = toolbar.getAttribute("ownertoolbox");
    let toolboxId = aToolboxId ? aToolboxId : ownertoolboxId ? ownertoolboxId :
                                                               gNavToolbox.id;
    if (toolbar.parentNode.id == toolboxId)
      return;
//TT._log("move tabstoolbar: "+toolboxId);
    let newToolbox = document.getElementById(toolboxId);
    let tabs = document.getElementById("tabbrowser-tabs");
    newToolbox.insertBefore(toolbar, newToolbox.lastChild);
    // Need to set the attribute, css doesn't work.
    // Note: vertical tabs positioning (contextmenu options) removed for now.
    if (newToolbox.hasAttribute("vertical")) {
      tabs.setAttribute("orient", "vertical");
      tabs.removeAttribute("flex");
      tabs.mTabstrip.setAttribute("orient", "vertical");
    }
    else {
      tabs.setAttribute("orient", "horizontal");
      tabs.setAttribute("flex", "1");
    }

    toolbar.setAttribute("ownertoolbox", newToolbox.id);
    document.persist(toolbar.id, "ownertoolbox");

    if (newToolbox.id == gNavToolbox.id) {
      if (TabsOnTop.enabled) {
        TabsOnTop.enabled = false;
        setTimeout(function() { TabsOnTop.toggle() }, 0);
      }
      else
        document.getElementById("main-window").removeAttribute("tabsontop");

      // Recreate externalToolbars list if moving a toolbar and remove it;
      // if it's added to a non navToolbox toolbox then the constructor will
      // add it back to the list upon append.
      let newExternalToolbars = [];
//      toolbar.removeAttribute("toolboxid");
      gNavToolbox.externalToolbars.forEach(function (extToolbar, index) {
        if (extToolbar.id != toolbar.id)
          newExternalToolbars.push(extToolbar);
      })
      gNavToolbox.externalToolbars = newExternalToolbars;
    }
    else {
      // New toolbox is not 'Top' location.  So if TabsOnTop is checked, and
      // they aren't there, the App Button will overflow due to its (odd) css
      // relative pos design.  So we remove the tabsontop attribute to avoid
      // this css rule.  Note that the checkbox will remain checked so if tabs
      // are moved back to 'Top' it will behave as where the user last left the
      // selection.
      document.getElementById("main-window").removeAttribute("tabsontop");

      // Add to external toolbars list for customization.
      gNavToolbox.externalToolbars.push(toolbar);
    }

    // Moved tabs toolbar; first tab title needs to be restored (4.0b8pre).
    let tab = gBrowser.tabs[0];
    tab.label = tab.linkedBrowser.contentTitle;
  },

  /* Set or unset fixed width for vertical toolbars */
  adjustToolboxWidth: function adjustToolboxWidth(fix) {
    // Aufruf durch Drag-Event der Toolbox.
    if (typeof fix == "object") {
      fix = false;
      window.setTimeout(function() { TT.adjustToolboxWidth(true) }, 200);
    }

    let cStyle, myWidth;
    let vToolboxes = document.getElementsByAttribute("vertical", "true");

    for (let i = 0; i < vToolboxes.length; i++) {
      if (vToolboxes[i].tagName == "toolbox") {
        myWidth = 0;
        // Set fixed width.
        if (fix) {
          let boxChilds = vToolboxes[i].childNodes;
          // Add up width of all toolbars in the toolbox.
          for (let j = 0; j < boxChilds.length; j++) {
            if (boxChilds[j].tagName == "toolbar") {
              cStyle = window.getComputedStyle(boxChilds[j], null);
              myWidth = myWidth + parseInt(cStyle.width);
              myWidth = myWidth + parseInt(cStyle.paddingLeft);
              myWidth = myWidth + parseInt(cStyle.paddingRight);
              myWidth = myWidth + parseInt(cStyle.marginLeft);
              myWidth = myWidth + parseInt(cStyle.marginRight);
              myWidth = myWidth + parseInt(cStyle.borderLeftWidth);
              myWidth = myWidth + parseInt(cStyle.borderRightWidth);
            }
          }

          vToolboxes[i].style.maxWidth = myWidth + "px";
        }
        // Unset width.
        else {
          vToolboxes[i].removeAttribute("width");
          vToolboxes[i].removeAttribute("style");
        }
      }
    }

    return true;
  },

  /* Set empty attribute on toolbox for fixed toolbars that are empty. */
  setFixedToolbarEmptyState: function setFixedToolbarEmptyState() {
    let fixedToolbars = document.getElementsByClassName("tt-toolbar-fixed");
    for (let i = 0; i < fixedToolbars.length; i++) {
      let fixedToolbar = fixedToolbars[i];
      if (fixedToolbar.parentNode.localName == "toolbox") {
        if (!fixedToolbar.childElementCount)
          fixedToolbar.parentNode.setAttribute("empty", true);
        else
          fixedToolbar.parentNode.removeAttribute("empty");
      }
    }
  },

  /**
   * Copy above and below browser toolbars to binding toolboxes if TreeStyleTabs
   * is installed and toolbars are in those locations.
   *
   * @param inCustomizeMode bool - if true, operate on binding toolboxes.
   */
  setToolbarsForTST: function setToolbarsForTST(inCustomizeMode) {
    if (!this.isTreeStyleTabsEnabled || !this.isTreeStyleTabsModeVertical)
      return;
//TT._log("inCustomizeMode - "+inCustomizeMode);

//TT._log("above");
    let movedToolbarIds = [];
    let fromToolboxID = inCustomizeMode ?
        "tt-toolbox-abovebrowser-treestyletab-mode-vertical" :
        "tt-toolbox-abovebrowser";
    let fromToolbox = document.getElementById(fromToolboxID);
    let toToolboxID = inCustomizeMode ?
        "tt-toolbox-abovebrowser" :
        "tt-toolbox-abovebrowser-treestyletab-mode-vertical";
    let toToolbox = document.getElementById(toToolboxID);
    let fromToolbars = Array.filter(fromToolbox.childNodes,
                                    function(toolbar) {
                                      return toolbar.localName == "toolbar";
                                    });
    if (fromToolbars.length) {
      fromToolbars.forEach(function(toolbar) {
//TT._log("id: "+toolbar.id);
        movedToolbarIds.push(toolbar.id);
        toToolbox.insertBefore(toolbar, toToolbox.lastChild);
      });
      toToolbox.removeAttribute("empty");
    }

//TT._log("below");
    fromToolboxID = inCustomizeMode ?
        "tt-toolbox-belowbrowser-treestyletab-mode-vertical" :
        "tt-toolbox-belowbrowser";
    fromToolbox = document.getElementById(fromToolboxID);
    toToolboxID = inCustomizeMode ?
        "tt-toolbox-belowbrowser" :
        "tt-toolbox-belowbrowser-treestyletab-mode-vertical";
    toToolbox = document.getElementById(toToolboxID);
    let fromToolbars = Array.filter(fromToolbox.childNodes,
                                    function(toolbar) {
                                      return toolbar.localName == "toolbar";
                                    });
    if (fromToolbars.length) {
      fromToolbars.forEach(function(toolbar) {
//TT._log("id: "+toolbar.id);
        movedToolbarIds.push(toolbar.id);
        toToolbox.insertBefore(toolbar, toToolbox.lastChild);
      });
      toToolbox.removeAttribute("empty");
    }
//TT._log("movedToolbarIds START: "+movedToolbarIds);

    if (!movedToolbarIds.length)
      // No above/below browser toolbars.
      return;

    // Recreate externalToolbars list..
    let newExternalToolbars = [];
    gNavToolbox.externalToolbars.forEach(function (extToolbar, index) {
//TT._log("extToolbar.id: "+extToolbar.id);
      if (movedToolbarIds.indexOf(extToolbar.id) == -1)
        newExternalToolbars.push(extToolbar);
      else
        // Only remove the first duplicate; constructor has already added the
        // second on append.  Remove the id from the moved list so as not to
        // remove the second.
        movedToolbarIds = movedToolbarIds.filter(function(id) {
                                                   return id != extToolbar.id;
                                                 });
//TT._log("movedToolbarIds: index:ids "+index+":"+movedToolbarIds);
    })
    gNavToolbox.externalToolbars = newExternalToolbars;
  },

  /* Set up toolbar context menu */
  initCustomiseContext: function initCustomiseContext(evt, popup) {
    let toolbar = document.popupNode;
    while (toolbar && toolbar.localName != "toolbar")
      toolbar = toolbar.parentNode;
    if (!toolbar)
      return;

    let isTabsToolbar         = toolbar.id == "TabsToolbar";
    let isFixedToolbar        = ["toolbar-menubar",
                                 "nav-bar",
                                 "PersonalToolbar",
                                 "addon-bar"].indexOf(toolbar.id) != -1;
    let isCustomizableToolbar = toolbar.getAttribute("customizable") == "true" ? true : false;
    let isFixedToolbarExtra   = toolbar.classList.contains("tt-toolbar-fixed");
    let child;

    // Einstellungen.
    let mode              = toolbar.getAttribute("mode");
    let lockmode          = toolbar.getAttribute("lockmode") == "true" ? true : false;
    let small             = toolbar.getAttribute("iconsize") == "small";
    let lockiconsize      = toolbar.getAttribute("lockiconsize") == "true" ? true : false;
    let flexbuttons       = toolbar.hasAttribute("flexbuttons");
    let fullscreentoolbar = toolbar.hasAttribute("fullscreentoolbar");
    let position          = toolbar.parentNode.getAttribute("id");

    for (let i = 0; i < popup.childNodes.length; i++) {
      child = popup.childNodes[i];
      // M O D E
      if (child.getAttribute("name") == "mode") {
        child.setAttribute("checked", child.getAttribute("value") == mode);
        child.disabled = lockmode;
      }
      // S M A L L I C O N S
      if (child.getAttribute("value") == "smallicons") {
        child.setAttribute("checked", small);
        child.disabled = mode == "text" || lockiconsize;
      }
      // F L E X I B L E   B U T T O N S
      if (child.getAttribute("value") == "flexbuttons") {
        child.setAttribute("checked", flexbuttons);
        // keine flexiblen Buttons in der Tabbar und der Statusleiste.
//      child.hidden = isFixedToolbox;
//      popup.childNodes[i-1].hidden = isFixedToolbox;
      }
      // F U L L S C R E E N
      if (child.getAttribute("value") == "fullscreentoolbar") {
        child.setAttribute("checked", fullscreentoolbar);
      }
      // P O S I T I O N
      if (child.getAttribute("name") == "position") {
        child.setAttribute("checked", child.getAttribute("value") == position);
        // Hidden for fixed toolboxes (sidebar header), if not customizable,
        // if a fixed toolbar (default toolbars), or vertical positions for
        // tabstoolbar, or if TreeStyleTabs is enabled.
        child.hidden = isFixedToolbar || isFixedToolbarExtra ||
                       !isCustomizableToolbar ||
                       (isTabsToolbar && child.hasAttribute("vertical")) ||
                       (isTabsToolbar && this.isTreeStyleTabsEnabled);
        // Separator hidden.
        if (child.id == "position1")
          popup.childNodes[i-1].hidden = child.hidden;

        // Special case for TabsToolbar.
        if ((child.value == "tt-toolbox-abovebrowsertabs" && !isTabsToolbar) ||
            (child.value == "tt-toolbox-abovebrowser" && isTabsToolbar))
         child.hidden = true;
      }
    }
  },

  /* Change individual toolbar configuration */
  setToolbarMode: function setToolbarMode(evt, aPopup) {
    let menuitem = evt.originalTarget;
    let toolbar = document.popupNode;
    while (toolbar.localName != "toolbar")
      toolbar = toolbar.parentNode;
    let isTabsToolbar = toolbar.id == "TabsToolbar";
    // All moveable toolbars have a toolbox parentNode.
    let toolbox = toolbar.parentNode;
    let removeToolbox = toolbox;
    switch (toolbox.id) {
      // For TST above/below browser binding toolboxes we need to operate on
      // the real toolbox.
      case "tt-toolbox-abovebrowser-treestyletab-mode-vertical":
        toolbox = document.getElementById("tt-toolbox-abovebrowser");
      break;
      case "tt-toolbox-belowbrowser-treestyletab-mode-vertical":
        toolbox = document.getElementById("tt-toolbox-belowbrowser");
      break;
      default:
        // No change.
    }
    let value = menuitem.value;
    let group = menuitem.getAttribute("name"); // radio group

    if (value == "smallicons") {
      let small = menuitem.getAttribute("checked") == "true";
      let size = small ? "small" : "large";
      toolbar.setToolbarProperty("iconsize", size);
    }
    else if (value == "fullscreentoolbar") {
      let showInFullScreen = menuitem.getAttribute("checked") == "true";
      toolbar.setToolbarProperty("fullscreentoolbar", showInFullScreen);
    }
    else if (value == "flexbuttons") {
      let flexbuttons = menuitem.getAttribute("checked") == "true";
      toolbar.setToolbarProperty("flexbuttons", flexbuttons);
    }
    else if (group == "mode") {
      toolbar.setToolbarProperty("mode", value);
    }
    else if (group == "position") {
      // Special case of TabsToolbar.
      if (isTabsToolbar) {
        TT.setTabsToolbar(value);
        TT.adjustToolboxWidth(aPopup);
        return;
      }

      // We could just remove the <toolbar/> from its current location and
      // append to the desired toolbox, but that would mean we wouldn't
      // respect the rule that custom toolbars appear wherever the
      // <toolbarset/> is.  Also it caused an interesting bug where all
      // the items on the toolbar would be duplicated sometimes after the
      // toolbar was moved.  Finally, we can't just get the currentSet of
      // the existing toolbar and pass it to appendCustomToolbar because
      // currentSet always returns "__empty" during customisation, because
      // all the items are wrapped in extra elements.
      let items = [];
      let currentSet = toolbar.currentSet;
      let tbarName = toolbar.toolbarName;
      let customIndex = toolbar.getAttribute("customindex");
      while (toolbar.hasChildNodes())
        items.push(toolbar.removeChild(toolbar.lastChild));
      if (toolbox.toolbarset.hasAttribute("toolbar"+customIndex)) {
        // Remove regular custom toolbar's toolbarset on the document root.
        toolbox.toolbarset.removeAttribute("toolbar"+customIndex);
        document.persist(toolbox.toolbarset.id, "toolbar"+customIndex);
      }

      removeToolbox.removeChild(toolbar);

      // Need to now clean up old toolbox: customToolbarCount, remaining
      // toolbar customindexes, toolbarset toolbarX numbering, and
      // persisted values.
      let customCount = 0;
      for (let i = 0; i < removeToolbox.childNodes.length; ++i) {
        let toolbarO = removeToolbox.childNodes[i];
        if (toolbarO.localName == "toolbar" &&
            toolbarO.getAttribute("customizable")) {
          // Calculate currentset and store it in the attribute.
          let currentSetO = toolbarO.currentSet;
          let customIndexO = toolbarO.getAttribute("customindex");
          if (customIndexO) {
            // Reindex custom toolbar info on the <toolbarset/>.
            if (toolbox.toolbarset.hasAttribute("toolbar"+customIndexO)) {
              toolbox.toolbarset.removeAttribute("toolbar"+customIndexO);
              document.persist(toolbox.toolbarset.id, "toolbar"+customIndexO);
            }
            toolbox.toolbarset.setAttribute("toolbar" + (++customCount),
                toolbarO.toolbarName + ":" + currentSetO);
            toolbarO.setAttribute("customindex", customCount); //adjust count
            document.persist(toolbox.toolbarset.id, "toolbar" + customCount);
          }
        }
      }
      // Number of remaining toolbars.
      toolbox.customToolbarCount = customCount;
      if (customCount == 0 && removeToolbox.id != "navigator-toolbox")
        // None..
        removeToolbox.setAttribute("empty", true);

      // Remove toolbarX attributes from toolbarset or xul doc root for
      // removed toolbars, ie any entries past those for toolbars already
      // indexed..
      while (toolbox.toolbarset.hasAttribute("toolbar" + (++customCount))) {
        toolbox.toolbarset.removeAttribute("toolbar" + customCount);
        document.persist(toolbox.toolbarset.id, "toolbar" + customCount);
      }

      // Recreate externalToolbars list..
      let newExternalToolbars = [];
//        toolbar.removeAttribute("toolboxid");
      gNavToolbox.externalToolbars.forEach(function (extToolbar, index) {
        if (extToolbar.id != toolbar.id)
          newExternalToolbars.push(extToolbar);
      })
      gNavToolbox.externalToolbars = newExternalToolbars;

      // Create new toolbar.
      let newToolbox = document.getElementById(value);
      let newToolbar = newToolbox.appendCustomToolbar(tbarName, "", true);
      // Append the items.
      while (items.length)
        newToolbar.appendChild(items.pop());
//TT._log("tbarName:newToolbar.toolbarName:cnt - "+
//        tbarName+":"+newToolbar.toolbarName+":"+newToolbar.childElementCount);

      // Retain options.
      TT.handleOptions("copy", newToolbar, gNavToolbox);
      customIndex = newToolbar.getAttribute("customindex");
      newToolbar.setAttribute("currentset", currentSet);
      newToolbox.removeAttribute("empty");

      // Persist custom toolbar info on the <toolbarset/>.  We have to use
      // tbarName as the toolbar.toolbarName getter doesn't always return a
      // value, bizarre.
      newToolbox.toolbarset.setAttribute("toolbar" + customIndex,
                                         tbarName + ":" + currentSet);
      document.persist(newToolbox.toolbarset.id, "toolbar" + customIndex);

      // If moving to above/below browser toolbox and have TST enabled with
      // vertical mode, do the move to the binding toolboxes.  We can't move
      // there directly as the state is persisted on the real toolbox
      // toolbarsets and there is no persistence on anon content elements.
      switch (value) {
        case "tt-toolbox-abovebrowser":
        case "tt-toolbox-belowbrowser":
          if (this.isTreeStyleTabsEnabled && this.isTreeStyleTabsModeVertical)
            setTimeout(function() { TT.setToolbarsForTST(false) }, 100);
        break;
        default:
          // Do nothing.
      }
    }

    TT.adjustToolboxWidth(aPopup);
  },

  /* Clean up options from root doc for removed or empty toolbars */
  handleOptions: function handleOptions(aType, toolbar, aNavToolbox) {
    if (toolbar.localName != "toolbar")
      return;
    let attr = "_toolbar." + toolbar.id + ".";
    let toolbarOptions = ["mode",
                          "iconsize",
                          "fullscreentoolbar",
                          "flexbuttons",
                          "collapsed"];

    var docElt = aNavToolbox.ownerDocument.documentElement;

    toolbarOptions.forEach(function(option) {
      if (docElt.hasAttribute(attr + option)) {
        if (aType == "remove") {
          if (toolbar.hasAttribute(option)) {
            // Remove global iconsize/mode options.
            toolbar.removeAttribute(option);
          }
          // Remove customtoolbar options.
          docElt.removeAttribute(attr + option);
          aNavToolbox.ownerDocument.persist(docElt.id, attr + option);
        }
        if (aType == "copy") {
          let val = docElt.getAttribute(attr + option);
          toolbar.setAttribute(option, val);
        }
      }
    });
  },

  /* Show or hide toolbars in fullscreen mode */
  BrowserFullScreen: function TT_BrowserFullScreen() {
    let toolboxes = [];
    // Get all vertical toolboxes in the array.
    let vToolboxes = document.getElementsByAttribute("vertical", "true");
    for (let i = 0; i < vToolboxes.length; i++) {
      if (vToolboxes[i].tagName == "toolbox") toolboxes.push(vToolboxes[i]);
    }

    // Toolboxen durchlaufen.
    for (let i = 0; i < toolboxes.length; i++) {
      let boxChilds = toolboxes[i].childNodes;
      let fsChilds = 0;

      // Child-Elemente durchlaufen (Toolbars).
      for (let j = 0; j < boxChilds.length; j++) {
        if (boxChilds[j].getAttribute("fullscreentoolbar") == "true")
          fsChilds++;
      }

      // If all toolbars hidden, make toolbox also hidden.
      if (window.fullScreen)
        toolboxes[i].setAttribute("hidden", !(fsChilds > 0));
      else
        toolboxes[i].setAttribute("hidden", false);
    }
  },

  /* Close toolbar button */
  closeToolbar: function closeToolbar(aToolbar) {
    if (this.windowType == "navigator:browser") {
      if (aToolbar.hasAttribute("customindex"))
        // Custom toolbars need collapsed state persisted on the document.
        aToolbar.setToolbarProperty("collapsed", true);
      else
        setToolbarVisibility(aToolbar, false);
    }
    else {
      if (aToolbar.hasAttribute("customindex"))
        // Custom toolbars need collapsed state persisted on the document.
        aToolbar.setToolbarProperty("collapsed", true);
      else {
        // Don't persist for a fixed toolbar in another window, may never be
        // able to reopen (without digging in .rdf).
        aToolbar.setAttribute("collapsed", true);
//      document.persist(aToolbar.id, "collapsed");
      }
    }
  },

  /* Console logger */
  _log: function(aMsg, aName) {
    let funcName = arguments.callee.caller.name;
    Cc["@mozilla.org/consoleservice;1"].
    getService(Ci.nsIConsoleService).
    logStringMessage("TotalToolbar." +
                     (funcName ? funcName : aName ? aName : "(anonymous)") +
                     ": " + aMsg);
  }

}

/* Initialize */
setTimeout(function() { TT.init() }, 1000);


//******************************************************************************
// Customizations of browser.js functions.
//******************************************************************************

function onViewToolbarsPopupShowing(aEvent, aInsertPoint) {
  var popup = aEvent.target;
  if (popup != aEvent.currentTarget)
    return;

  // Empty the menu
  for (var i = popup.childNodes.length-1; i >= 0; --i) {
    var deadItem = popup.childNodes[i];
    if (deadItem.hasAttribute("toolbarId"))
      popup.removeChild(deadItem);
  }

   var firstMenuItem = aInsertPoint || popup.firstChild;

  let toolbarNodes = Array.slice(gNavToolbox.childNodes);
  toolbarNodes = toolbarNodes.concat(gNavToolbox.externalToolbars); //!!
  toolbarNodes.forEach(function(toolbar) {
    var toolbarName = toolbar.getAttribute("toolbarname");
    if (toolbarName) {
      let menuItem = document.createElement("menuitem");
      let hidingAttribute = toolbar.getAttribute("type") == "menubar" ?
                            "autohide" : "collapsed";
      menuItem.setAttribute("id", "toggle_" + toolbar.id);
      menuItem.setAttribute("toolbarId", toolbar.id);
      menuItem.setAttribute("type", "checkbox");
      menuItem.setAttribute("label", toolbarName);
      menuItem.setAttribute("checked",
                            toolbar.getAttribute(hidingAttribute) != "true");
      if (popup.id != "appmenu_customizeMenu")
        menuItem.setAttribute("accesskey", toolbar.getAttribute("accesskey"));
      popup.insertBefore(menuItem, firstMenuItem);

      menuItem.addEventListener("command", onViewToolbarCommand, false);
    }
  }, this);
}

/* Handle and persist collapsed state of custom toolbars, not just fixed ones.*/
function onViewToolbarCommand(aEvent) {
  let toolbarId = aEvent.originalTarget.getAttribute("toolbarId");
  let toolbar = document.getElementById(toolbarId);
  let isVisible = aEvent.originalTarget.getAttribute("checked") == "true";

  if (toolbar.hasAttribute("customindex"))
    // Custom toolbars need collapsed state persisted on the document.
    toolbar.setToolbarProperty("collapsed", !isVisible);
  else
    setToolbarVisibility(toolbar, isVisible);

  TT.adjustToolboxWidth(true);
}
